@@ -5,6 +5,8 @@ import copy
 import json
 import logging
 import shutil
+import shlex
+from string import Template
 
 LOGGER = logging.getLogger('playbook_runner')
 
@@ -31,7 +33,10 @@ class AnsiblePlaybook(object):
             'all'
         )
 
-        self._hosts.add(*hosts)
+        hosts.append('localhost')
+        hosts.append('127.0.0.1')
+
+        self._hosts.update(hosts)
 
         LOGGER.info('Output path: {0}'.format(self._path_str))
         LOGGER.info('CWD: {0}'.format(self._ansible_playbook_directory))
@@ -74,6 +79,8 @@ class AnsiblePlaybook(object):
             ansible_command.insert(-1, '--extra-vars')
             ansible_command.insert(-1, extra_vars)
 
+        ansible_command = [shlex.quote(s) for s in ansible_command]
+
         return ansible_command
 
     def _get_hosts_by_group(self, inventory, group):
@@ -124,12 +131,26 @@ class AnsiblePlaybook(object):
 
         return hosts_in_group
 
+    def _generate_parent_play(self, dir_path, play_filename):
+        playbook_path = os.path.join(dir_path, play_filename)
+        tmplt_path = os.path.join(os.path.dirname(__file__), 'parent_play.tmplt')
+        parent_path = '{0}/{1}.json'.format(self._path_str, play_filename + '.parent')
+
+        with open(tmplt_path, "r") as tmplt_file:
+            tmplt = Template(tmplt_file.read())
+
+        with open(parent_path, 'w') as parent_file:
+            parent_file.write(tmplt.substitute({"PLAYBOOK_PATH": playbook_path}))
+
+        return parent_path
+
+
     def run_playbook(self, play_filename, extra_vars_dict=None):
         if not extra_vars_dict:
             extra_vars_dict = {}
 
         local_extra_vars = copy.deepcopy(extra_vars_dict)
-        local_extra_vars['output_path'] = self._path_str
+        local_extra_vars['playbooks_output_path'] = self._path_str
         if 'skip_errors' not in local_extra_vars:
             local_extra_vars['skip_errors'] = False
         if 'gather_facts_for_pb' not in local_extra_vars:
@@ -137,10 +158,10 @@ class AnsiblePlaybook(object):
         if 'play_host_groups' not in local_extra_vars:
             local_extra_vars['play_host_groups'] = 'localhost'
 
+        path = self._generate_parent_play(self._ansible_playbook_directory, play_filename)
         cmd = self._get_ansible_cmd(
             self._ansible_playbook_inventory,
-            '{0}/{1}'.format(
-                self._ansible_playbook_directory, play_filename),
+            path,
             extra_vars_dict=local_extra_vars)
 
         for host in self._hosts:
@@ -155,13 +176,23 @@ class AnsiblePlaybook(object):
         ansible_output_path = '{0}/ansible_output_path.txt'.format(
             self._path_str
         )
+
+        our_env = os.environ.copy()
+        our_env["ANSIBLE_HOST_KEY_CHECKING"] = "False"
         with open(ansible_output_path, "a+") as f_ansible_output_path:
+            f_ansible_output_path.write(
+                '\n\nGoing to run ansible playbook:\n{0}\n\n'.format(
+                    ' '.join(cmd)
+                    )
+            )
+
             result = subprocess.run(
                 cmd,
                 cwd=self._ansible_playbook_directory,
                 timeout=120,
                 stdout=f_ansible_output_path,
                 stderr=subprocess.STDOUT,
+                env=our_env
             )
 
             if not local_extra_vars['skip_errors']:
@@ -182,15 +213,18 @@ class AnsiblePlaybook(object):
                 data = lines
 
             # format file into a correct json format
-            with open(file_path, "w+") as f:
-                last_line = data[len(data) - 1]
-                last_line = last_line[:-3]
-                data[len(data) - 1] = last_line
+            with open(file_path, "w") as f:
+                last_line = data[-1]
+                last_line = last_line.strip()[:-1]
+                data[-1] = last_line
                 f.writelines(data)
                 f.write(']')
 
             with open(file_path) as f:
-                last_output[host] = json.load(f)
+                try:
+                    last_output[host] = json.load(f)
+                except Exception:
+                    pass
 
             os.remove(file_path)
 
